package fr.mael.jiwigo.dao;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.jdom.Document;
import org.jdom.Element;

import fr.mael.jiwigo.om.Category;
import fr.mael.jiwigo.transverse.enumeration.CategoryEnum;
import fr.mael.jiwigo.transverse.enumeration.MethodsEnum;
import fr.mael.jiwigo.transverse.exception.ProxyAuthenticationException;
import fr.mael.jiwigo.transverse.session.SessionManager;
import fr.mael.jiwigo.transverse.util.Tools;

/*
 *  jiwigo-ws-api Piwigo webservice access Api
 *  Copyright (c) 2010-2011 Mael mael@le-guevel.com
 *                All Rights Reserved
 *
 *  This library is free software. It comes without any warranty, to
 *  the extent permitted by applicable law. You can redistribute it
 *  and/or modify it under the terms of the Do What The Fuck You Want
 *  To Public License, Version 2, as published by Sam Hocevar. See
 *  http://sam.zoy.org/wtfpl/COPYING for more details.
 */
/**
 * Dao for the categories
 * @author mael
 *
 */
public class CategoryDao {
    /**
     * Logger
     */
    public static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory
	    .getLog(CategoryDao.class);
    /**
     * Instance to use a singleton
     */
    private static CategoryDao instance;

    private SessionManager sessionManager;

    /**
     * Private constructor to use a singleton
     */
    private CategoryDao(SessionManager sessionManager) {
	this.sessionManager = sessionManager;
    }

    /**
     * @return the instance
     */
    public static CategoryDao getInstance(SessionManager sessionManager) {
	if (instance == null) {
	    instance = new CategoryDao(sessionManager);
	}
	return instance;
    }

    /**
     * Lists the categories
     * @param recursive true : recursive listing of the categories
     * @return the list of categories
     * @throws IOException
     * @throws ProxyAuthenticationException
     */
    public List<Category> list(boolean recursive) throws IOException, ProxyAuthenticationException {
	Document doc = sessionManager.executeReturnDocument(MethodsEnum.LISTER_CATEGORIES.getLabel(), "recursive",
		String.valueOf(recursive));
	Element element = doc.getRootElement().getChild("categories");
	List<Element> listElement = (List<Element>) element.getChildren("category");
	ArrayList<Category> categories = new ArrayList<Category>();
	for (Element cat : listElement) {
	    Category myCat = new Category();
	    myCat.setIdentifier(Integer.valueOf(cat.getAttributeValue(CategoryEnum.ID.getLabel())));
	    myCat.setUrlCategory(cat.getAttributeValue(CategoryEnum.URL.getLabel()));
	    myCat.setNbImages(Integer.valueOf(cat.getAttributeValue(CategoryEnum.NB_IMAGES.getLabel())));
	    myCat.setNbTotalImages(Integer.valueOf(cat.getAttributeValue(CategoryEnum.NB_TOTAL_IMAGES.getLabel())));
	    myCat.setName(cat.getChildText(CategoryEnum.NAME.getLabel()));
	    String catMeres = cat.getChildText(CategoryEnum.CAT_MERES.getLabel());
	    ArrayList<Integer> idCategoriesMeres = new ArrayList<Integer>();
	    myCat.setIdParentCategoriesString(catMeres);
	    for (String catA : catMeres.split(",")) {
		if (!catA.equals("")) {
		    idCategoriesMeres.add(Integer.valueOf(catA));
		}
	    }
	    myCat.setIdParentCategories(idCategoriesMeres);
	    categories.add(myCat);
	}
	return categories;
    }

    /**
     * Creation of a category
     * @param category the category to create
     * @return true if the category is successfully created
     * @throws ProxyAuthenticationException
     */
    public boolean create(Category category) throws ProxyAuthenticationException {
	try {
	    if (category.getDirectParent() != null) {
		return Tools.checkOk(sessionManager.executeReturnDocument(MethodsEnum.AJOUTER_CATEGORIE.getLabel(),
			"name", category.getName(), "parent", String.valueOf(category.getDirectParent())));
	    } else {
		return Tools.checkOk(sessionManager.executeReturnDocument(MethodsEnum.AJOUTER_CATEGORIE.getLabel(),
			"name", category.getName()));
	    }
	} catch (IOException e) {
	    LOG.error(Tools.getStackTrace(e));
	}
	return false;
    }

}
