package fr.mael.jiwigo.dao;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.jdom.Document;
import org.jdom.Element;

import fr.mael.jiwigo.om.Comment;
import fr.mael.jiwigo.transverse.enumeration.MethodsEnum;
import fr.mael.jiwigo.transverse.exception.ProxyAuthenticationException;
import fr.mael.jiwigo.transverse.session.SessionManager;
import fr.mael.jiwigo.transverse.util.Tools;

/*
 *  jiwigo-ws-api Piwigo webservice access Api
 *  Copyright (c) 2010-2011 Mael mael@le-guevel.com
 *                All Rights Reserved
 *
 *  This library is free software. It comes without any warranty, to
 *  the extent permitted by applicable law. You can redistribute it
 *  and/or modify it under the terms of the Do What The Fuck You Want
 *  To Public License, Version 2, as published by Sam Hocevar. See
 *  http://sam.zoy.org/wtfpl/COPYING for more details.
 */
/**

 * Dao for the comments
 * @author mael
 *
 */
public class CommentDao {
    /**
     * Logger
     */
    public static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory
	    .getLog(CommentDao.class);
    /**
     *  Instance that allows to use a singleton
     */
    private static CommentDao instance;

    private SessionManager sessionManager;

    /**
     * private constructor, to use a singleton
     */
    private CommentDao(SessionManager sessionManager) {
	this.sessionManager = sessionManager;
    }

    /**
     * @return the singleton
     */
    public static CommentDao getInstance(SessionManager sessionManager) {
	if (instance == null) {
	    instance = new CommentDao(sessionManager);
	}
	return instance;
    }

    /**
     * Listing of the comments for the given image
     * @param idImage id of the  image
     * @return list of comments
     * @throws IOException
     * @throws ProxyAuthenticationException
     */
    public List<Comment> list(Integer idImage) throws IOException, ProxyAuthenticationException {
	Document doc = (sessionManager.executeReturnDocument(MethodsEnum.GET_INFO.getLabel(), "image_id", String
		.valueOf(idImage), "comments_per_page", "100"));
	Element element = doc.getRootElement().getChild("image").getChild("comments");
	List<Element> listElement = (List<Element>) element.getChildren("comment");
	ArrayList<Comment> comments = new ArrayList<Comment>();
	for (Element com : listElement) {
	    Comment myCom = new Comment();
	    myCom.setIdentifier(Integer.valueOf(com.getAttributeValue("id")));
	    myCom.setDate(com.getAttributeValue("date"));
	    myCom.setAuthor(com.getChildText("author"));
	    myCom.setContent(com.getChildText("content"));
	    comments.add(myCom);
	}
	return comments;
    }

    /**
     * Gets the key essential to add a comment
     * @param idImage the id of the image
     * @return the key
     * @throws IOException
     * @throws ProxyAuthenticationException
     */
    public String getKey(Integer idImage) throws IOException, ProxyAuthenticationException {
	Document doc = (sessionManager.executeReturnDocument(MethodsEnum.GET_INFO.getLabel(), "image_id", String
		.valueOf(idImage)));
	String key = doc.getRootElement().getChild("image").getChild("comment_post").getAttributeValue("key");
	return key;
    }

    /**
     * Creates a comment
     * @param commentaire the comment to add
     * @return true if the comment is successfully added
     * @throws IOException
     * @throws ProxyAuthenticationException
     */
    public boolean create(Comment commentaire) throws IOException, ProxyAuthenticationException {
	String key = getKey(commentaire.getImageId());
	try {
	    Thread.sleep(2200);
	} catch (InterruptedException e) {
	    e.printStackTrace();
	}
	return Tools.checkOk(sessionManager.executeReturnDocument(MethodsEnum.AJOUTER_COMMENTAIRE.getLabel(),
		"image_id", String.valueOf(commentaire.getImageId()), "author", commentaire.getAuthor(), "content",
		commentaire.getContent(), "key", key));

    }
}
