package fr.mael.jiwigo.dao;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.jdom.Document;
import org.jdom.Element;

import fr.mael.jiwigo.om.Image;
import fr.mael.jiwigo.om.Tag;
import fr.mael.jiwigo.transverse.enumeration.MethodsEnum;
import fr.mael.jiwigo.transverse.enumeration.TagEnum;
import fr.mael.jiwigo.transverse.exception.ProxyAuthenticationException;
import fr.mael.jiwigo.transverse.session.SessionManager;
import fr.mael.jiwigo.transverse.util.Tools;

/*
 *  jiwigo-ws-api Piwigo webservice access Api
 *  Copyright (c) 2010-2011 Mael mael@le-guevel.com
 *                All Rights Reserved
 *
 *  This library is free software. It comes without any warranty, to
 *  the extent permitted by applicable law. You can redistribute it
 *  and/or modify it under the terms of the Do What The Fuck You Want
 *  To Public License, Version 2, as published by Sam Hocevar. See
 *  http://sam.zoy.org/wtfpl/COPYING for more details.
 */
/**
 * Dao of the categories
 * @author mael
 *
 */
public class TagDao {
    /**
     * Logger
     */
    public static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(TagDao.class);
    /**
     * Instance, to use a singleton
     */
    private static TagDao instance;

    private SessionManager sessionManager;

    /**
     * private constructor to use a singleton
     */
    private TagDao(SessionManager sessionManager) {
	this.sessionManager = sessionManager;
    }

    /**
     * @return the instance
     */
    public static TagDao getInstance(SessionManager sessionManager) {
	if (instance == null) {
	    instance = new TagDao(sessionManager);
	}
	return instance;
    }

    /**
     * lists the tags
     * @return the list of tags
     * @throws IOException
     * @throws ProxyAuthenticationException
     */
    public List<Tag> list() throws IOException, ProxyAuthenticationException {
	Document doc = sessionManager.executeReturnDocument(MethodsEnum.TAGS_ADMIN_LIST.getLabel());
	//	System.out.println(Outil.documentToString(doc));
	return getTagsFromDocument(doc.getRootElement().getChild("tags"));

    }

    /**
     * COnstructs a list of tags from a document
     * @param doc the document
     * @return the list of tags
     */
    private List<Tag> getTagsFromDocument(Element element) {

	List<Element> listElement = (List<Element>) element.getChildren("tag");
	ArrayList<Tag> tags = new ArrayList<Tag>();
	for (Element tagElement : listElement) {
	    Tag tag = new Tag();
	    tag.setIdentifier(Integer.valueOf(tagElement.getAttributeValue(TagEnum.ID.getLabel())));
	    tag.setName(tagElement.getAttributeValue(TagEnum.NAME.getLabel()));
	    tags.add(tag);
	}
	return tags;

    }

    /**
     * Creation of a tag
     * @param tag the tag to create
     * @return true if the tag as been successfully created
     * @throws ProxyAuthenticationException
     */
    public boolean create(Tag tag) throws ProxyAuthenticationException {
	try {
	    return Tools.checkOk(sessionManager.executeReturnDocument(MethodsEnum.ADD_TAG.getLabel(), "name", tag
		    .getName()));
	} catch (IOException e) {
	    LOG.error(Tools.getStackTrace(e));
	}
	return false;
    }

    /**
     * Function that returns the tags for an image
     * @param image the image
     * @return the tags list
     * @throws IOException
     * @throws ProxyAuthenticationException
     */
    public List<Tag> tagsForImage(Image image) throws IOException, ProxyAuthenticationException {
	Document doc = sessionManager.executeReturnDocument(MethodsEnum.GET_INFO.getLabel(), "image_id", String
		.valueOf(image.getIdentifier()));
	return getTagsFromDocument(doc.getRootElement().getChild("image").getChild("tags"));
    }

}
